@@ -63,7 +63,7 @@ Hello %(firstname-subject)s %(surname-subject)s,
 thanks for registering on %(base_url)s.
 
 Please click on the link below to activate your account :
-%(url)s.
+%(url)s
 
 See you soon on %(base_url)s !
 ''')
@@ -132,11 +132,25 @@ class RegistrationConfirmController(controller.Controller):
         except:
             msg = req._(u'Invalid registration data. Please try registering again.')
             raise Redirect(req.build_url(u'register', __message=msg))
+        if self._cw.user.login == login:
+            # already logged in (e.g. regstration link replayed twice in the browser)
+            raise Redirect(self.success_redirect_url())
         req.form = data # hijack for proper validation error handling
-        self.appli.repo.register_user(login, password,
-                                      email=data.get(qname('address')),
-                                      firstname=data.get(qname('firstname')),
-                                      surname=data.get(qname('surname')))
+        err_raised = False
+        try:
+            self.appli.repo.register_user(login, password,
+                                          email=data.get(qname('address')),
+                                          firstname=data.get(qname('firstname')),
+                                          surname=data.get(qname('surname')))
+        except ValidationError, err:
+            err_raised = True
+            # XXX TEMPORARY HACK to allow registration links to work more than
+            # once. This is required because some email clients (e.g. kmail)
+            # start by downloading the url to find the mimetype of the resource
+            # and then execute the appropriate action (e.g. open the url in the
+            # default browser) based on the mimetype.
+            if err.errors.keys() != ['login-subject']:
+                raise
         # force new authentication (anon until there)
         clear_cache(req, 'get_authorization')
         req.form['__login'] = login
@@ -148,13 +162,18 @@ class RegistrationConfirmController(controller.Controller):
             self.appli.session_handler.set_session(req)
         except Redirect:
             pass
+        if req.user.login != 'login' and err_raised:
+            # if both authentication and register_user() failed, the problem
+            # is probably that we do have a true login conflict, just re-raise
+            # the original validation error
+            raise
         assert req.user.login == login
-        raise Redirect(self.success_redirect_url(data))
+        raise Redirect(self.success_redirect_url())
 
-    def success_redirect_url(self, data):
+    def success_redirect_url(self):
         msg = self._cw._(u'Congratulations, your registration is complete. '
-                         'Welcome %(firstname-subject)s %(surname-subject)s !')
-        return self._cw.build_url('', __message=msg%data)
+                         'Welcome %s !')
+        return self._cw.build_url('', __message=msg % self._cw.user.name())
 
 
 class AnonUserLink(basecomponents.AnonUserLink):
